Package: factR
Title: Functional Annotation of Custom Transcriptomes
Version: 1.15.0
Authors@R: person("Fursham", "Hamid", 
    email = "fursham.h@gmail.com", 
    role = c("aut", "cre"))
Description: factR contain tools to process and interact with custom-assembled 
    transcriptomes (GTF). At its core, factR constructs CDS information on 
    custom transcripts and subsequently predicts its functional output. 
    In addition,  factR has tools capable of plotting transcripts, 
    correcting chromosome and gene information and shortlisting new transcripts.
Depends: R (>= 4.2)
biocViews: AlternativeSplicing, FunctionalPrediction, GenePrediction
Imports: BiocGenerics (>= 0.46), Biostrings (>= 2.68), GenomeInfoDb (>=
        1.36), dplyr (>= 1.1), GenomicFeatures (>= 1.52), GenomicRanges
        (>= 1.52), IRanges (>= 2.34), purrr (>= 1.0), rtracklayer (>=
        1.60), tidyr (>= 1.3), methods (>= 4.3), BiocParallel (>=
        1.34), S4Vectors (>= 0.38), data.table (>= 1.14), rlang (>=
        1.1), tibble (>= 3.2), wiggleplotr (>= 1.24), RCurl (>= 1.98),
        XML (>= 3.99), drawProteins (>= 1.20), ggplot2 (>= 3.4),
        stringr (>= 1.5), pbapply (>= 1.7), stats (>= 4.3), utils (>=
        4.3), graphics (>= 4.3), crayon (>= 1.5)
License: file LICENSE
Encoding: UTF-8
ByteCompile: true
RoxygenNote: 7.2.1
Suggests: AnnotationHub (>= 2.22), BSgenome (>= 1.58),
        BSgenome.Mmusculus.UCSC.mm10, testthat, knitr, rmarkdown,
        markdown, zeallot, rmdformats, bio3d (>= 2.4), signalHsmm (>=
        1.5), tidyverse (>= 1.3), covr, patchwork
VignetteBuilder: knitr
LazyData: FALSE
BiocType: Software
URL: https://fursham-h.github.io/factR/
git_url: https://git.bioconductor.org/packages/factR
git_branch: devel
git_last_commit: 4d6b2ca
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:37:54 UTC; biocbuild
Author: Fursham Hamid [aut, cre]
Maintainer: Fursham Hamid <fursham.h@gmail.com>
